@@ -1,6 +1,6 @@
 #######################################################################
 # seriation - Infrastructure for seriation
-# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
+# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +19,13 @@
 
 
 ## Criterion generic.
-criterion <- function(x, order = NULL, method = NULL, ...)
-  UseMethod("criterion")
+criterion <-
+  function(x,
+    order = NULL,
+    method = NULL,
+    force_loss = FALSE,
+    ...)
+    UseMethod("criterion")
 
 ## Criterion method registry.
 
@@ -32,39 +37,73 @@ criterion <- function(x, order = NULL, method = NULL, ...)
 ## </NOTE>
 
 
-set_criterion_method <- function(kind, name, fun,
-  description = NULL, merit = NA, ...) {
+set_criterion_method <- function(kind,
+  name,
+  fun,
+  description = NULL,
+  merit = NA,
+  ...) {
   ## check formals
   ##if(!identical(names(formals(definition)),
   ##              c("x", "order", "...")))
   ##    stop("Criterion methods must have formals 'x', 'order', and '...'.")
 
   ## check if criterion is already in registry
-  r <- registry_criterion$get_entry(kind=kind, name=name)
-  if(!is.null(r) && r$name==name) {
+  r <- registry_criterion$get_entry(kind = kind, name = name)
+  if (!is.null(r) && r$name == name) {
     warning("Entry with name ", name, " already exists! Modifying entry.")
-    registry_criterion$modify_entry(kind=kind, name=name, fun=fun,
-      description=description, merit = merit)
+    registry_criterion$modify_entry(
+      kind = kind,
+      name = name,
+      fun = fun,
+      description = description,
+      merit = merit
+    )
   } else {
     registry_criterion$set_entry(
-      kind = kind, name=name, fun = fun,
-      description = description, merit = merit)
+      kind = kind,
+      name = name,
+      fun = fun,
+      description = description,
+      merit = merit
+    )
   }
 
 }
 
 get_criterion_method <- function(kind, name) {
-  method <- registry_criterion$get_entry(kind=kind, name=name)
-  if(is.null(method))
-    stop("Unknown seriation method. Check list_criterion_methods(\"", kind, "\")")
+  method <- registry_criterion$get_entry(kind = kind, name = name)
+  if (is.null(method))
+    stop("Unknown criterion. Check list_criterion_methods(\"",
+      kind,
+      "\")")
 
   method
 }
 
-list_criterion_methods <- function(kind)
-  sort(as.vector(sapply(registry_criterion$get_entries(kind=kind), "[[", "name")))
+list_criterion_methods <- function(kind) {
+  if (missing(kind)) {
+    kinds <- unique(sort(as.vector(
+      sapply(registry_criterion$get_entries(), "[[", "kind")
+    )))
+
+    sapply(
+      kinds,
+      FUN = function(k)
+        list_criterion_methods(k)
+    )
+
+  } else{
+    sort(as.vector(sapply(
+      registry_criterion$get_entries(kind = kind), "[[", "name"
+    )))
+  }
+}
 
 show_criterion_methods <- function(kind) {
-  m <- registry_criterion$get_entries(kind=kind)
+  if (missing(kind))
+    m <- registry_criterion$get_entries()
+  else
+    m <- registry_criterion$get_entries(kind = kind)
   m[sort(names(m))]
 }
